from importlib.util import find_spec
from pathlib import Path
from typing import Optional
from typing import Union


# See https://github.com/psf/requests/issues/1081#issuecomment-428504128
class ForceMultipartDict(dict):
    def __bool__(self) -> bool:
        return True


def guess_mime_type_stdlib(url: Union[str, Path]) -> Optional[str]:  # pragma: no cover
    """
    Guesses the MIME type of a URL using the standard library.

    Args:
        url: The URL to guess the MIME type for.

    Returns:
        The guessed MIME type, or None if it could not be determined.
    """

    import mimetypes

    mime_type, _ = mimetypes.guess_type(str(url))  # Ensure URL is a string
    return mime_type


def guess_mime_type_magic(url: Union[str, Path]) -> Optional[str]:
    """
    Guesses the MIME type of a file using libmagic.

    Args:
        url: The path to the file or URL to guess the MIME type for.

    Returns:
        The guessed MIME type, or None if it could not be determined.
    """

    import magic  # type: ignore[import-not-found]

    try:
        return magic.from_file(str(url), mime=True)  # type: ignore[misc]
    except Exception:  # pragma: no cover
        # Handle libmagic exceptions gracefully
        return None


# Use the best option
guess_mime_type = guess_mime_type_magic if find_spec("magic") is not None else guess_mime_type_stdlib



import os
import sys
current_dir = os.getcwd()
sys.path.append(current_dir + "/src")

import logging
from office_to_pdf_client import OfficeToPdfClient, OfficeToPdfClientAsync
from unittest.mock import Mock  # Import Mock for patching
from pytest_mock import MockerFixture
from pathlib import Path


def test_init_default_values():
    """
    Test that the OfficeToPdfClient is initialized with default values.
    """
    client = OfficeToPdfClient("http://127.0.0.1:8000")
    assert client._client.base_url == "http://127.0.0.1:8000"
    assert client._client.timeout.read == 30.0
    assert client.http2 is True  # httpxのclientから抽出できなかったので、OfficeToPdfClientのpropertyから取得できるように設定
    assert logging.getLogger("httpx").level == logging.ERROR
    assert logging.getLogger("httpcore").level == logging.ERROR


def test_init_custom_values():
    """
    Test that the OfficeToPdfClient is initialized with custom values.
    """
    client = OfficeToPdfClient(
        "http://127.0.0.1:8000", timeout=10.0, log_level=logging.INFO, http2=False
    )
    assert client._client.base_url == "http://127.0.0.1:8000"
    assert client._client.timeout.read == 10.0
    assert client.http2 is False
    assert logging.getLogger("httpx").level == logging.INFO
    assert logging.getLogger("httpcore").level == logging.INFO


def test_add_headers():
    """
    Test that the add_headers method updates the client headers.
    """
    client = OfficeToPdfClient("http://127.0.0.1:8000")
    headers = {"Content-Type": "application/json"}
    set_headers = [
        (
            b'Accept',
            b'*/*',
        ),
        (
            b'Accept-Encoding',
            b'gzip, deflate',
        ),
        (
            b'Connection',
            b'keep-alive',
        ),
        (
            b'User-Agent',
            b'python-httpx/0.28.1',
        ),
        (
            b'Content-Type',
            b'application/json',
        ),
    ]
    client.add_headers(headers)
    assert client._client.headers.raw == set_headers


def test_get_resource(mocker: MockerFixture):
    # モックの作成
    get_resource_mock = mocker.patch("office_to_pdf_client.OfficeToPdfClient._get_resource")
    get_resource_mock.return_value = {"file": "test"}

    # 入力のパスを設定
    input_file_path = Path('input.txt')

    # テスト対象の関数呼び出し
    converter = OfficeToPdfClient("http://127.0.0.1:8000")  # 実際のクラスに置き換える
    result = converter._get_resource(input_file_path)

    # アサーション
    assert result == {"file": "test"}


def test_convert_to_pdf_success(mocker):
    # モックの作成
    convert_to_pdf_mock = mocker.patch("office_to_pdf_client.OfficeToPdfClient.convert_to_pdf")
    convert_to_pdf_mock.return_value = None

    # 入力と出力のパスを設定
    input_file_path = Path('input.txt')
    output_file_path = Path('output.pdf')

    # テスト対象の関数呼び出し
    converter = OfficeToPdfClient("http://127.0.0.1:8000")  # 実際のクラスに置き換える
    result = converter.convert_to_pdf(input_file_path, output_file_path)

    # アサーション
    assert result is None


def test_convert_to_pdf_success_async(mocker):
    # モックの作成
    convert_to_pdf_mock = mocker.patch("office_to_pdf_client.OfficeToPdfClientAsync.convert_to_pdf")
    convert_to_pdf_mock.return_value = None

    # 入力と出力のパスを設定
    input_file_path = Path('input.txt')
    output_file_path = Path('output.pdf')

    # テスト対象の関数呼び出し
    converter = OfficeToPdfClientAsync("http://127.0.0.1:8000")  # 実際のクラスに置き換える
    result = converter.convert_to_pdf(input_file_path, output_file_path)


def test_close():
    """
    Test that the close method closes the underlying HTTP client connection.
    """
    # Mock the httpx.Client close method
    client = OfficeToPdfClient("http://127.0.0.1:8000")
    client._client.close = Mock()
    client.close()
    client._client.close.assert_called_once()



import logging
from contextlib import ExitStack
from pathlib import Path
from types import TracebackType
from typing import Optional

from httpx import AsyncClient, Client
from httpx._types import RequestFiles

from office_to_pdf_client._utils import guess_mime_type


logger = logging.getLogger(__name__)


class OfficeToPdfClient:
    def __init__(
        self,
        host: str,
        *,
        timeout: float = 30.0,
        log_level: int = logging.ERROR,
        http2: bool = True,
        api_route: str = "/convert_to_pdf"
    ):
        """_summary_
        Initialize a new office-to-pdf instance.

        Args:
            host (str): _description_
            timeout (float, optional): _description_. Defaults to 30.0.
            log_level (int, optional): _description_. Defaults to logging.ERROR.
            http2 (bool, optional): _description_. Defaults to True.
            api_route (str, optional): _description_. Defaults to /convert_to_pdf.
        """
        # Configure the client
        self._http2 = http2
        self._route = api_route
        self._stack = ExitStack()
        self._client = Client(base_url=host, timeout=timeout, http2=http2)

        # Set the log level
        logging.getLogger("httpx").setLevel(log_level)
        logging.getLogger("httpcore").setLevel(log_level)

    @property
    def http2(self):
        return self._http2

    def add_headers(self, header: dict[str, str]) -> None:
        """
        Update the httpx Client headers with the given values.

        Args:
            header (Dict[str, str]): A dictionary of header names and values to add.
        """
        self._client.headers.update(header)

    def _get_resource(self, filepath: Path) -> RequestFiles:
        """
        Deals with opening all provided files for multi-part uploads, including
        pushing their new contexts onto the stack to ensure resources like file
        handles are cleaned up
        """
        resource = {}
        filename = filepath.name
        # Helpful but not necessary to provide the mime type when possible
        mime_type = guess_mime_type(filepath)
        if mime_type is not None:
            resource.update(
                {"file": (filename, self._stack.enter_context(filepath.open("rb")), mime_type)},
            )
        else:  # pragma: no cover
            resource.update({"file": (filename, self._stack.enter_context(filepath.open("rb")))})  # type: ignore [dict-item]
        return resource

    def convert_to_pdf(
        self,
        input_file_path: Path,
        output_file_path: Path,
        sheet_names: list[str] | None = None,
        single_page_sheets: bool = False,
    ) -> None:
        """
        convert a single file to PDF.

        Args:
            input_file_path (Path): The path to the file to be converted.
            output_file_path (Path): The path to the file for output.
            sheet_names (list[str] | None): The names of the sheets to be converted (optional).
            single_page_sheets (bool): Whether to convert the sheets to single page sheets (optional).

        Returns:
            None
        """

        response = self._client.post(
            url=self._route,
            files=self._get_resource(input_file_path),
            data={"sheet_names": sheet_names} if sheet_names is not None else None,
            params={"single_page_sheets": single_page_sheets},
        )
        response.raise_for_status()
        output_file_path.write_bytes(response.content)

    def close(self) -> None:
        """
        Close the underlying HTTP client connection.
        """
        self._client.close()

    def __exit__(
        self,
        exc_type: Optional[type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        """
        Exit the runtime context related to this object.

        This method ensures that the client connection is closed when exiting a context manager.

        Args:
            exc_type: The type of the exception that caused the context to be exited, if any.
            exc_val: The instance of the exception that caused the context to be exited, if any.
            exc_tb: A traceback object encoding the stack trace, if an exception occurred.
        """
        self.close()


class OfficeToPdfClientAsync(OfficeToPdfClient):
    def __init__(
        self,
        host: str,
        *,
        timeout: float = 30.0,
        log_level: int = logging.ERROR,
        http2: bool = True,
        api_route: str = "/convert_to_pdf"
    ):
        super().__init__(host, timeout=timeout, log_level=log_level, http2=http2, api_route=api_route)
        self._client = AsyncClient(base_url=host, timeout=timeout, http2=http2)

    async def convert_to_pdf(
        self,
        input_file_path: Path,
        output_file_path: Path,
        sheet_names: list[str] | None = None,
        single_page_sheets: bool = False,
    ) -> None:
        response = await self._client.post(
            url=self._route,
            files=self._get_resource(input_file_path),
            data={"sheet_names": sheet_names} if sheet_names is not None else None,
            params={"single_page_sheets": single_page_sheets},
        )
        response.raise_for_status()
        output_file_path.write_bytes(response.content)

    async def close(self) -> None:
        await self._client.aclose()
